# Copyright (c) 2011 OpenStack, LLC.
# Copyright (c) 2012 Cloudscaling
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from nova import flags
from nova.openstack.common import cfg
from nova.openstack.common import log as logging
from nova.scheduler import filters

LOG = logging.getLogger(__name__)

ram_allocation_ratio_opt = cfg.FloatOpt("ram_allocation_ratio",
        default=1.5,
        help="virtual ram to physical ram allocation ratio")

FLAGS = flags.FLAGS
FLAGS.register_opt(ram_allocation_ratio_opt)


class RamFilter(filters.BaseHostFilter):
    """Ram Filter with over subscription flag"""

    def host_passes(self, host_state, filter_properties):
        """Only return hosts with sufficient available RAM."""
        instance_type = filter_properties.get('instance_type')
        requested_ram = instance_type['memory_mb']
        free_ram_mb = host_state.free_ram_mb
        total_usable_ram_mb = host_state.total_usable_ram_mb

        oversubscribed_ram_limit_mb = (total_usable_ram_mb *
                FLAGS.ram_allocation_ratio)
        used_ram_mb = total_usable_ram_mb - free_ram_mb
        usable_ram = oversubscribed_ram_limit_mb - used_ram_mb
        if not usable_ram >= requested_ram:
            LOG.debug(_("%(host_state)s does not have %(requested_ram)s MB "
                    "usable ram, it only has %(usable_ram)s MB usable ram."),
                    locals())
            return False

        # save oversubscribe ram limit so the compute host can verify
        # memory availability on builds:
        filter_properties['memory_limit_mb'] = oversubscribed_ram_limit_mb
        return True
